@@ -19,7 +19,6 @@
 
 import os.path
 import sys
-#import httplib2
 import urllib2
 from urllib import urlencode
 import pycurl
@@ -37,7 +36,7 @@ import clam.common.formats
 from clam.common.data import CLAMData, CLAMFile, CLAMInputFile, CLAMOutputFile, CLAMMetaData, InputTemplate, OutputTemplate, VERSION as DATAAPIVERSION, BadRequest, NotFound, PermissionDenied, ServerError, AuthRequired,NoConnection, UploadError, ParameterError, TimeOut, processhttpcode
 from clam.common.util import RequestWithMethod
 
-VERSION = '0.9.10'
+VERSION = '0.9.11'
 if VERSION != DATAAPIVERSION:
     raise Exception("Version mismatch beween Client API ("+clam.common.data.VERSION+") and Data API ("+DATAAPIVERSION+")!")
 
@@ -46,23 +45,28 @@ register_openers()
 
 
 class CLAMClient:
-    def __init__(self, url, user=None, password=None):
+    def __init__(self, url, user=None, password=None, oauth=False, oauth_access_token=None):
         """Initialise the CLAM client (does not actually connect yet)
 
         * ``url`` - URL of the webservice
-        * ``user`` - username (or None if no authentication is needed)
-        * ``password`` - password (or None if not authentication is needed)
+        * ``user`` - username (or None if no authentication is needed or if using OAuth2)
+        * ``password`` - password (or None if no authentication is needed or if using OAuth2)
+        * ``oauth`` - Use OAuth2? (boolean)
+        * ``oauth_access_token`` - OAuth2 Access Token (or None), if OAuth is
+            enabled and no token is specified, the authorization provider will be called to obtain one.
+            If this stage requires user interaction, it will fail.
         """
 
         #self.http = httplib2.Http()
         if url[-1] != '/': url += '/'
         self.url = url
+        self.oauth = oauth
+        self.oauth_access_token = oauth_access_token
         if user and password:
             self.authenticated = True
-            #for most things we use httplib2, no we don't anymore
-            #self.http.add_credentials(user, password)
             self.user = user
             self.password = password
+            self.oauth = False
             self.initauth()
         else:
             self.authenticated = False
@@ -83,6 +87,40 @@ class CLAMClient:
             opener = urllib2.build_opener(authhandler)
             opener.addheaders = [('User-agent', 'CLAMClientAPI-' + VERSION)]
             urllib2.install_opener(opener)
+        elif self.oauth:
+            if not self.oauth_access_token:
+                opener = urllib2.build_opener()
+                opener.addheaders = [('User-agent', 'CLAMClientAPI-' + VERSION)]
+                urllib2.install_opener(opener)
+                request = RequestWithMethod(self.url,method='GET')
+                try:
+                    response = urllib2.urlopen(request)
+                except Exception, e:
+                    try:
+                        statuscode = int(e.code)
+                    except AttributeError:
+                        raise e
+
+                    if statuscode >= 200 and statuscode < 300:
+                        #this is no error!
+                        return self._parse(e.read())
+                    elif statuscode == 404:
+                        raise NotFound("Authorization provider not found")
+                    elif statuscode == 403:
+                        raise PermissionDenied("Authorization provider denies access")
+                    else:
+                        raise
+
+                data = self._parse(response.read())
+                if data is True: #indicates failure
+                    raise Exception("No access token provided, but Authorization Provider requires manual user input. Unable to authenticate automatically. Obtain an access token from " + response.geturl())
+                else:
+                    self.oauth_access_token = data.oauth_access_token
+
+            opener = urllib2.build_opener()
+            opener.addheaders = [('User-agent', 'CLAMClientAPI-' + VERSION)]
+            opener.addheaders = [('Authorization', 'Bearer ' + self.oauth_access_token)]
+            urllib2.install_opener(opener)
         else:
             opener = urllib2.build_opener()
             opener.addheaders = [('User-agent', 'CLAMClientAPI-' + VERSION)]
@@ -94,7 +132,7 @@ class CLAMClient:
     def request(self, url='', method = 'GET', data = None):
         """Issue a HTTP request and parse CLAM XML response, this is a low-level function called by all of the higher-level communicaton methods in this class, use those instead"""
 
-        if self.authenticated: self.initauth()
+        if self.authenticated or self.oauth: self.initauth()
         if data:
             request = RequestWithMethod(self.url + url,data,method=method)
         else:
@@ -171,6 +209,19 @@ class CLAMClient:
         return self.request(project + '/', 'PUT')
 
 
+
+    def action(self, id, **kwargs):
+        """Query an action, specify the parameters for the action as keyword parameters. An optional keyword parameter method='GET' (default) or method='POST' can be set."""
+        if 'method' in kwargs:
+            method = kwargs['method']
+            del kwargs['method']
+        else:
+            method = 'GET'
+
+        return self.request('/actions/' + id, method, urlencode(kwargs))
+
+
+
     def start(self, project, **parameters):
         """Start a run. ``project`` is the ID of the project, and ``parameters`` are keyword arguments for
         the global parameters. Returns a ``CLAMData`` object or raises exceptions. Note that no exceptions are raised on parameter errors, you have to check for those manually! (Use startsafe instead if want Exceptions on parameter errors)::
@@ -377,8 +428,11 @@ class CLAMClient:
         c.setopt(pycurl.URL, self.url + project + '/input/' + filename)
         fields = list(data.items())
         c.setopt(c.HTTPPOST, fields)
-        c.setopt(c.HTTPAUTH, c.HTTPAUTH_DIGEST)
-        c.setopt(c.USERPWD, self.user + ':' + self.password)
+        if self.authenticated:
+            c.setopt(c.HTTPAUTH, c.HTTPAUTH_DIGEST)
+            c.setopt(c.USERPWD, self.user + ':' + self.password)
+        elif self.oauth:
+            c.setopt(c.HTTPHEADER, [ 'Authorization: Bearer %s' % str(self.oauth_access_token) ])
         c.setopt(c.WRITEFUNCTION, buf.write)
         c.perform()
         code = processhttpcode(c.getinfo(c.HTTP_CODE),[403]) #raises exception when not successful
