@@ -6,23 +6,7 @@ function initclam() {
         window.location.reload(); //alert("System error: data.js not properly loaded?");
    }
 
-   if (typeof(stage) != 'undefined') {
-       //Download parameters.xsl so it's available to javascript for file-parameters
-       $.ajax({ 
-            type: "GET", 
-            url: baseurl + "/static/parameters.xsl",
-            dataType: "xml", 
-            success: function(xml){ 
-                parametersxsl = xml;
-            }
-       });
-      if (stage == 1) {
-       		$('#progressbar').progressbar({value: progress});
-       		setTimeout(pollstatus,2000);
-       }
-    }
 
-   
    //set custom text, this hack is needed because XSLT 1.0 can't get the job
    //done alone in all browsers
    if (($('#customhtml').length > 0) && $('#customhtml').html().search("&lt;") != 1) {
@@ -89,9 +73,15 @@ function initclam() {
         }
         if (duplicate) continue;
         processed.push(inputtemplates[i].id);     
-        inputtemplate_options += '<option value="' + inputtemplates[i].id + '">' + inputtemplates[i].label + '</option>';
+        if ((i == 0) && (preselectinputtemplate) ) {
+            selected='selected="selected"';
+        } else {
+            selected="";
+        }
+        inputtemplate_options += '<option value="' + inputtemplates[i].id + '" ' + selected + '>' + inputtemplates[i].label + '</option>';
    }
    $(".inputtemplates").html(inputtemplate_options);
+   
 
    //Tying events to trigger rendering of file-parameters when an inputtemplate is selected:
    $("#uploadinputtemplate").change(function(event){renderfileparameters($('#uploadinputtemplate').val(),'#uploadparameters',true); });
@@ -106,6 +96,32 @@ function initclam() {
         }
     });
 
+   if (typeof(stage) != 'undefined') {
+       //Download parameters.xsl so it's available to javascript for file-parameters
+       $.ajax({ 
+            type: "GET", 
+            url: baseurl + "/static/parameters.xsl",
+            dataType: "xml", 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
+            success: function(xml){ 
+                parametersxsl = xml;
+                if (preselectinputtemplate) {
+                  //we have a preselection, trigger
+                  $("#uploadinputtemplate").trigger('change');
+                  $("#urluploadinputtemplate").trigger('change');
+                  $("#editorinputtemplate").trigger('change');
+                }
+            }
+       });
+      if (stage == 1) {
+       		$('#progressbar').progressbar({value: progress});
+       		setTimeout(pollstatus,2000);
+       }
+    }
 
 
    //Create a new project'
@@ -117,8 +133,17 @@ function initclam() {
             type: "PUT", 
             url: baseurl + '/' + $("#projectname").val() + "/", 
             dataType: "text", 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
             success: function(response){ 
-                window.location.href = baseurl + '/' + $("#projectname").val() + "/";
+                if (oauth_access_token != "") {
+                  window.location.href = baseurl + '/' + $("#projectname").val() + "/?oauth_access_token="+oauth_access_token;
+                } else {
+                  window.location.href = baseurl + '/' + $("#projectname").val() + "/";
+                }
             },
             error: function(response){
                 if ((response.status < 200) || (response.status > 299)) { //patch
@@ -137,8 +162,17 @@ function initclam() {
             type: "DELETE", 
             url: baseurl + '/' + project + '/', 
             dataType: "text", 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
             success: function(response){ 
-                window.location.href = baseurl + '/'; /* back to index */
+                if (oauth_access_token != "") {
+                  window.location.href = baseurl + '/?oauth_access_token=' + oauth_access_token; /* back to index */
+                } else {
+                  window.location.href = baseurl + '/'; /* back to index */
+                }
             },
             error: function(response){
                 if ((response.status < 200) || (response.status > 299)) { //patch
@@ -156,6 +190,11 @@ function initclam() {
             type: "DELETE", 
             url: baseurl + '/' + project + "/output/" , 
             dataType: "text", 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
             success: function(xml){ 
                 window.location.href = ""; /* refresh */
             },
@@ -171,7 +210,11 @@ function initclam() {
    //Return to index
    if ($("#indexbutton").length) {
        $("#indexbutton").click(function(event){
-            window.location.href = baseurl + '/'; /* refresh */
+            if (oauth_access_token != "") {
+              window.location.href = baseurl + '/?oauth_access_token=' + oauth_access_token; /* back to index */
+            } else {
+              window.location.href = baseurl + '/'; /* back to index */
+            }
        });
    }  
    
@@ -226,6 +269,11 @@ function initclam() {
             url: baseurl + '/' + project + "/input/" + filename, 
             dataType: "xml", 
             data: data, 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
             success: function(response){            
                 processuploadresponse(response, '#editorparameters');
                 $('#editorcontents').val('');
@@ -257,6 +305,11 @@ function initclam() {
                 url: baseurl + '/' + project + "/input/" + filename, 
                 dataType: "xml", 
                 data: {'url': $('#urluploadfile').val(), 'inputtemplate': $('#urluploadinputtemplate').val() }, 
+                beforeSend: oauthheader,
+                crossDomain: true,
+                xhrFields: {
+                  withCredentials: true
+                },
                 success: function(response){
                     processuploadresponse(response, '#urluploadparameters');
                     $('#urluploadprogress').hide();                     
@@ -296,7 +349,11 @@ function initclam() {
 		        return true;
 		}).on('complete', function(event, id, fileName, responseJSON) {
 				if (responseJSON.isarchive) {
-					window.location.href = baseurl + '/' + project + '/'; /* just refresh page */
+                    if (oauth_access_token != "") {
+                      window.location.href = baseurl + '/' + project + "/?oauth_access_token="+oauth_access_token;
+                    } else {
+                      window.location.href = baseurl + '/' + project + "/";
+                    }
 					return true;
 				} 
 		        processuploadresponse(responseJSON.xml, '#uploadparameters');
@@ -344,8 +401,17 @@ function initclam() {
                 url: baseurl + '/' + project + "/input/", 
                 //dataType: "xml", 
                 data: {'inputsource': $('#inputsource').val() }, 
+                beforeSend: oauthheader,
+                crossDomain: true,
+                xhrFields: {
+                  withCredentials: true
+                },
                 success: function(response){
-                    window.location.href = baseurl + '/' + project + '/'; /* refresh */   
+                    if (oauth_access_token != "") {
+                      window.location.href = baseurl + '/' + project + "/?oauth_access_token="+oauth_access_token;
+                    } else {
+                      window.location.href = baseurl + '/' + project + "/";
+                    }
                 },
                 error: function(response, errortype){
                     alert(response);
@@ -360,15 +426,28 @@ function initclam() {
             type: "POST",
             url: baseurl + '/' + project + "/input/",
             data: {'inputsource': $('#uploadinputsource').val() }, 
+            beforeSend: oauthheader,
+            crossDomain: true,
+            xhrFields: {
+              withCredentials: true
+            },
             success: function(response){
                 //processuploadresponse(response, '#nonexistant');
-                window.location.href = baseurl + '/' + project + '/'; /* refresh */   
-            },
+                if (oauth_access_token != "") {
+                  window.location.href = baseurl + '/' + project + "/?oauth_access_token="+oauth_access_token;
+                } else {
+                  window.location.href = baseurl + '/' + project + "/";
+                }
+          },
             error: function(response,errortype){
                 $('#inputsourceprogress').hide();
                 $('#inputsourceupload').show();
                 if ((response.status >= 200) && (response.status <= 299)) { //patch
-                    window.location.href = baseurl + '/' + project + '/'; /* refresh */   
+                    if (oauth_access_token != "") {
+                      window.location.href = baseurl + '/' + project + "/?oauth_access_token="+oauth_access_token;
+                    } else {
+                      window.location.href = baseurl + '/' + project + "/";
+                    }
                 } else {
                     alert("Error, unable to add file ("+response.status+")");
                 }
@@ -588,6 +667,11 @@ function deleteinputfile(filename) {
     if (found >= 0) tableinputfiles.fnDeleteRow(found);
     $.ajax({ 
         type: "DELETE", 
+        beforeSend: oauthheader,
+        crossDomain: true,
+        xhrFields: {
+          withCredentials: true
+        },
         url: baseurl + '/' + project + "/input/" + filename, 
         dataType: "xml"
     });    
@@ -618,11 +702,20 @@ function pollstatus() {
 	$.ajax({
 		type: 'GET',
 		url: baseurl + '/' + project + "/status/",
+        beforeSend: oauthheader,
+        crossDomain: true,
+        xhrFields: {
+          withCredentials: true
+        },
 		dataType: 'json', 
 		data: {accesstoken: accesstoken, user: user},
 		success: function(response){
                 if (response.statuscode != 1) {
-                	window.location.href = baseurl + '/' + project + '/'; /* refresh */
+                    if (oauth_access_token != "") {
+                      window.location.href = baseurl + '/' + project + '/?oauth_access_token=' + oauth_access_token; /* refresh */
+                    } else {
+                      window.location.href = baseurl + '/' + project + '/'; /* refresh */
+                    }
                 } else {
                 	if (response.completion > 0) {
                 		progress = response.completion;
@@ -644,3 +737,8 @@ function pollstatus() {
 	});	
 }
 
+function oauthheader(req) { 
+  if (oauth_access_token != "") {
+    req.setRequestHeader("Authorization", "Bearer " + oauth_access_token);
+  }
+}
